import { Promotion, RequestContext } from '@vendure/core';
import { StripeSubscriptionPricing, StripeSubscriptionPricingInput, SubscriptionInterval, SubscriptionStartMoment } from '../ui/generated/graphql';
import { Schedule } from './schedule.entity';
import { OrderLineWithSubscriptionFields, VariantWithSubscriptionFields } from './subscription-custom-fields';
export type VariantForCalculation = Pick<VariantWithSubscriptionFields, 'id' | 'listPrice' | 'customFields'>;
/**
 * Calculate subscription pricing based on variants, schedules and optional input
 * Doesn't apply discounts yet!
 *
 * @param ctx
 * @param rawSubscriptionPrice
 * @param schedule The schedule that should be used for the Subscription
 * @param input
 * @returns
 */
export declare function calculateSubscriptionPricing(ctx: RequestContext, rawSubscriptionPrice: number, schedule: Schedule, input?: Pick<StripeSubscriptionPricingInput, 'downpayment' | 'startDate'>): Omit<StripeSubscriptionPricing, 'variantId' | 'originalRecurringPrice'>;
/**
 * Calculate the discounted recurring price based on given promotions
 */
export declare function applySubscriptionPromotions(ctx: RequestContext, recurringPrice: number, orderLine: OrderLineWithSubscriptionFields, promotions: Promotion[]): Promise<number>;
/**
 * Calculate day rate based on the total price and duration of the subscription
 * Example: $200 per 6 months
 *          = $400 per 12 months
 *          $400 / 365 = $1,10 per day
 */
export declare function getDayRate(totalPrice: number, durationInterval: SubscriptionInterval, durationCount: number): number;
export declare function getDaysUntilNextStartDate(now: Date, nextStartDate: Date): number;
/**
 * Get the next startDate for a given start moment (first or last of the Interval). Always returns the middle of the day for billing
 */
export declare function getNextStartDate(now: Date, interval: SubscriptionInterval, startMoment: SubscriptionStartMoment, fixedStartDate?: Date): Date;
/**
 * Get the next cycles startDate. Used for paid-up-front subscriptions, where a user already paid for the first cycle
 * and we need the next cycles start date
 */
export declare function getNextCyclesStartDate(now: Date, startMoment: SubscriptionStartMoment, interval: SubscriptionInterval, intervalCount: number, fixedStartDate?: Date): Date;
/**
 * Get the endDate of a subscription based on the duration
 */
export declare function getEndDate(startDate: Date, startMoment: SubscriptionStartMoment, interval: SubscriptionInterval, intervalCount: number): Date;
/**
 * Return the middle of the day (13:00) for dates, because that makes more sense for billing
 */
export declare function getMiddleOfDay(date: Date): Date;
/**
 * Get the number of billings per full duration of the schedule
 */
export declare function getBillingsPerDuration(schedule: Pick<Schedule, 'durationInterval' | 'durationCount' | 'billingInterval' | 'billingCount'>): number;
/**
 * Yes, it's real, this helper function prints money for you!
 */
export declare function printMoney(amount: number): string;
export declare function cloneSchedule(ctx: RequestContext, schedule: Schedule): Schedule & {
    pricesIncludeTax: boolean;
};
