"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScheduleService = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@vendure/core");
const graphql_1 = require("../ui/generated/graphql");
const pricing_helper_1 = require("./pricing.helper");
const schedule_entity_1 = require("./schedule.entity");
let ScheduleService = class ScheduleService {
    constructor(connection) {
        this.connection = connection;
    }
    async getSchedules(ctx) {
        const schedules = await this.connection
            .getRepository(ctx, schedule_entity_1.Schedule)
            .find({ where: { channelId: String(ctx.channelId) } });
        return schedules.map((schedule) => {
            return (0, pricing_helper_1.cloneSchedule)(ctx, schedule);
        });
    }
    async upsert(ctx, input) {
        this.validate(input);
        const { id } = await this.connection.getRepository(ctx, schedule_entity_1.Schedule).save({
            ...input,
            channelId: String(ctx.channelId),
        });
        const schedule = await this.connection
            .getRepository(ctx, schedule_entity_1.Schedule)
            .findOneOrFail({ where: { id } });
        return (0, pricing_helper_1.cloneSchedule)(ctx, schedule);
    }
    async delete(ctx, scheduleId) {
        const { id } = await this.connection.rawConnection
            .getRepository(schedule_entity_1.Schedule)
            .findOneOrFail({
            where: {
                id: scheduleId,
                channelId: ctx.channelId,
            },
        });
        await this.connection.getRepository(ctx, schedule_entity_1.Schedule).delete({ id });
    }
    validate(input) {
        if (input.billingInterval === input.durationInterval &&
            input.billingCount === input.durationCount &&
            input.downpayment) {
            throw new core_1.UserInputError(`Paid up front schedules can not have downpayments. When duration and billing intervals are the same your schedule is a paid-up-front schedule.`);
        }
        if (input.startMoment === graphql_1.SubscriptionStartMoment.FixedStartdate &&
            !input.fixedStartDate) {
            throw new core_1.UserInputError(`Schedules with a fixed start date require a selected startDate`);
        }
        if (input.startMoment === graphql_1.SubscriptionStartMoment.FixedStartdate &&
            input.useProration) {
            throw new core_1.UserInputError(`Schedules with a fixed start date cannot use proration`);
        }
    }
};
ScheduleService = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [core_1.TransactionalConnection])
], ScheduleService);
exports.ScheduleService = ScheduleService;
