import { ID, OrderService, ProductService, RequestContext } from '@vendure/core';
import { Request } from 'express';
import { StripeSubscriptionPluginOptions } from '../stripe-subscription.plugin';
import { StripeSubscriptionPricing, StripeSubscriptionPricingInput, StripeSubscriptionSchedule, UpsertStripeSubscriptionScheduleInput } from '../ui/generated/graphql';
import { ScheduleService } from './schedule.service';
import { StripeSubscriptionService } from './stripe-subscription.service';
import { IncomingStripeWebhook } from './stripe.types';
import { OrderLineWithSubscriptionFields } from './subscription-custom-fields';
export type RequestWithRawBody = Request & {
    rawBody: any;
};
export declare class ShopResolver {
    private stripeSubscriptionService;
    private orderService;
    private productService;
    constructor(stripeSubscriptionService: StripeSubscriptionService, orderService: OrderService, productService: ProductService);
    createStripeSubscriptionIntent(ctx: RequestContext): Promise<string>;
    stripeSubscriptionPricing(ctx: RequestContext, input: StripeSubscriptionPricingInput): Promise<StripeSubscriptionPricing>;
    stripeSubscriptionPricingForProduct(ctx: RequestContext, productId: ID): Promise<StripeSubscriptionPricing[]>;
}
export declare class ShopOrderLinePricingResolver {
    private subscriptionService;
    constructor(subscriptionService: StripeSubscriptionService);
    subscriptionPricing(ctx: RequestContext, orderLine: OrderLineWithSubscriptionFields): Promise<StripeSubscriptionPricing | undefined>;
}
export declare class AdminPriceIncludesTaxResolver {
    pricesIncludeTax(ctx: RequestContext, orderLine: OrderLineWithSubscriptionFields): boolean;
}
export declare class AdminResolver {
    private scheduleService;
    constructor(scheduleService: ScheduleService);
    stripeSubscriptionSchedules(ctx: RequestContext): Promise<StripeSubscriptionSchedule[]>;
    upsertStripeSubscriptionSchedule(ctx: RequestContext, input: UpsertStripeSubscriptionScheduleInput): Promise<StripeSubscriptionSchedule>;
    deleteStripeSubscriptionSchedule(ctx: RequestContext, scheduleId: string): Promise<void>;
}
export declare class StripeSubscriptionController {
    private stripeSubscriptionService;
    private orderService;
    private options;
    constructor(stripeSubscriptionService: StripeSubscriptionService, orderService: OrderService, options: StripeSubscriptionPluginOptions);
    webhook(signature: string | undefined, request: RequestWithRawBody, body: IncomingStripeWebhook): Promise<void>;
}
