"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StripeSubscriptionController = exports.AdminResolver = exports.AdminPriceIncludesTaxResolver = exports.ShopOrderLinePricingResolver = exports.ShopResolver = void 0;
const common_1 = require("@nestjs/common");
const graphql_1 = require("@nestjs/graphql");
const core_1 = require("@vendure/core");
const constants_1 = require("../constants");
const schedule_service_1 = require("./schedule.service");
const stripe_subscription_service_1 = require("./stripe-subscription.service");
let ShopResolver = class ShopResolver {
    constructor(stripeSubscriptionService, orderService, productService) {
        this.stripeSubscriptionService = stripeSubscriptionService;
        this.orderService = orderService;
        this.productService = productService;
    }
    async createStripeSubscriptionIntent(ctx) {
        return this.stripeSubscriptionService.createPaymentIntent(ctx);
    }
    async stripeSubscriptionPricing(ctx, input) {
        return this.stripeSubscriptionService.getPricingForVariant(ctx, input);
    }
    async stripeSubscriptionPricingForProduct(ctx, productId) {
        const product = await this.productService.findOne(ctx, productId, [
            'variants',
        ]);
        if (!product) {
            throw new core_1.UserInputError(`No product with id '${productId}' found`);
        }
        const subscriptionVariants = product.variants.filter((v) => !!v.customFields.subscriptionSchedule && v.enabled);
        return await Promise.all(subscriptionVariants.map((variant) => this.stripeSubscriptionService.getPricingForVariant(ctx, {
            productVariantId: variant.id,
        })));
    }
};
__decorate([
    (0, graphql_1.Mutation)(),
    (0, core_1.Allow)(core_1.Permission.Owner),
    __param(0, (0, core_1.Ctx)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [core_1.RequestContext]),
    __metadata("design:returntype", Promise)
], ShopResolver.prototype, "createStripeSubscriptionIntent", null);
__decorate([
    (0, graphql_1.Query)(),
    __param(0, (0, core_1.Ctx)()),
    __param(1, (0, graphql_1.Args)('input')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [core_1.RequestContext, Object]),
    __metadata("design:returntype", Promise)
], ShopResolver.prototype, "stripeSubscriptionPricing", null);
__decorate([
    (0, graphql_1.Query)(),
    __param(0, (0, core_1.Ctx)()),
    __param(1, (0, graphql_1.Args)('productId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [core_1.RequestContext, Object]),
    __metadata("design:returntype", Promise)
], ShopResolver.prototype, "stripeSubscriptionPricingForProduct", null);
ShopResolver = __decorate([
    (0, graphql_1.Resolver)(),
    __metadata("design:paramtypes", [stripe_subscription_service_1.StripeSubscriptionService,
        core_1.OrderService,
        core_1.ProductService])
], ShopResolver);
exports.ShopResolver = ShopResolver;
let ShopOrderLinePricingResolver = class ShopOrderLinePricingResolver {
    constructor(subscriptionService) {
        this.subscriptionService = subscriptionService;
    }
    async subscriptionPricing(ctx, orderLine) {
        if (orderLine.productVariant?.customFields?.subscriptionSchedule) {
            return await this.subscriptionService.getPricingForOrderLine(ctx, orderLine);
        }
        return;
    }
};
__decorate([
    (0, graphql_1.ResolveField)(),
    __param(0, (0, core_1.Ctx)()),
    __param(1, (0, graphql_1.Parent)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [core_1.RequestContext, Object]),
    __metadata("design:returntype", Promise)
], ShopOrderLinePricingResolver.prototype, "subscriptionPricing", null);
ShopOrderLinePricingResolver = __decorate([
    (0, graphql_1.Resolver)('OrderLine'),
    __metadata("design:paramtypes", [stripe_subscription_service_1.StripeSubscriptionService])
], ShopOrderLinePricingResolver);
exports.ShopOrderLinePricingResolver = ShopOrderLinePricingResolver;
// This is needed to resolve schedule.pricesIncludeTax in the Admin UI
let AdminPriceIncludesTaxResolver = class AdminPriceIncludesTaxResolver {
    pricesIncludeTax(ctx, orderLine) {
        return ctx.channel.pricesIncludeTax;
    }
};
__decorate([
    (0, graphql_1.ResolveField)(),
    __param(0, (0, core_1.Ctx)()),
    __param(1, (0, graphql_1.Parent)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [core_1.RequestContext, Object]),
    __metadata("design:returntype", Boolean)
], AdminPriceIncludesTaxResolver.prototype, "pricesIncludeTax", null);
AdminPriceIncludesTaxResolver = __decorate([
    (0, graphql_1.Resolver)('StripeSubscriptionSchedule')
], AdminPriceIncludesTaxResolver);
exports.AdminPriceIncludesTaxResolver = AdminPriceIncludesTaxResolver;
let AdminResolver = class AdminResolver {
    constructor(scheduleService) {
        this.scheduleService = scheduleService;
    }
    async stripeSubscriptionSchedules(ctx) {
        return this.scheduleService.getSchedules(ctx);
    }
    async upsertStripeSubscriptionSchedule(ctx, input) {
        return this.scheduleService.upsert(ctx, input);
    }
    async deleteStripeSubscriptionSchedule(ctx, scheduleId) {
        return this.scheduleService.delete(ctx, scheduleId);
    }
};
__decorate([
    (0, core_1.Allow)(core_1.Permission.ReadSettings),
    (0, graphql_1.Query)(),
    __param(0, (0, core_1.Ctx)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [core_1.RequestContext]),
    __metadata("design:returntype", Promise)
], AdminResolver.prototype, "stripeSubscriptionSchedules", null);
__decorate([
    (0, core_1.Allow)(core_1.Permission.UpdateSettings),
    (0, graphql_1.Mutation)(),
    __param(0, (0, core_1.Ctx)()),
    __param(1, (0, graphql_1.Args)('input')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [core_1.RequestContext, Object]),
    __metadata("design:returntype", Promise)
], AdminResolver.prototype, "upsertStripeSubscriptionSchedule", null);
__decorate([
    (0, core_1.Allow)(core_1.Permission.UpdateSettings),
    (0, graphql_1.Mutation)(),
    __param(0, (0, core_1.Ctx)()),
    __param(1, (0, graphql_1.Args)('scheduleId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [core_1.RequestContext, String]),
    __metadata("design:returntype", Promise)
], AdminResolver.prototype, "deleteStripeSubscriptionSchedule", null);
AdminResolver = __decorate([
    (0, graphql_1.Resolver)(),
    __metadata("design:paramtypes", [schedule_service_1.ScheduleService])
], AdminResolver);
exports.AdminResolver = AdminResolver;
let StripeSubscriptionController = class StripeSubscriptionController {
    constructor(stripeSubscriptionService, orderService, options) {
        this.stripeSubscriptionService = stripeSubscriptionService;
        this.orderService = orderService;
        this.options = options;
    }
    async webhook(signature, request, body) {
        core_1.Logger.info(`Incoming webhook ${body.type}`, constants_1.loggerCtx);
        // Validate if metadata present
        const orderCode = body.data.object.metadata?.orderCode ||
            body.data.object.lines?.data[0]?.metadata.orderCode;
        const channelToken = body.data.object.metadata?.channelToken ||
            body.data.object.lines?.data[0]?.metadata.channelToken;
        if (body.type !== 'payment_intent.succeeded' &&
            body.type !== 'invoice.payment_failed' &&
            body.type !== 'invoice.payment_succeeded') {
            core_1.Logger.info(`Received incoming '${body.type}' webhook, not processing this event.`, constants_1.loggerCtx);
            return;
        }
        if (!orderCode) {
            return core_1.Logger.error(`Incoming webhook is missing metadata.orderCode, cannot process this event`, constants_1.loggerCtx);
        }
        if (!channelToken) {
            return core_1.Logger.error(`Incoming webhook is missing metadata.channelToken, cannot process this event`, constants_1.loggerCtx);
        }
        try {
            const ctx = await this.stripeSubscriptionService.createContext(channelToken, request);
            const order = await this.orderService.findOneByCode(ctx, orderCode);
            if (!order) {
                throw Error(`Cannot find order with code ${orderCode}`); // Throw inside catch block, so Stripe will retry
            }
            // Validate signature
            const { stripeClient } = await this.stripeSubscriptionService.getStripeHandler(ctx, order.id);
            if (!this.options?.disableWebhookSignatureChecking) {
                stripeClient.validateWebhookSignature(request.rawBody, signature);
            }
            if (body.type === 'payment_intent.succeeded') {
                await this.stripeSubscriptionService.handlePaymentIntentSucceeded(ctx, body, order);
            }
            else if (body.type === 'invoice.payment_succeeded') {
                await this.stripeSubscriptionService.handleInvoicePaymentSucceeded(ctx, body, order);
            }
            else if (body.type === 'invoice.payment_failed') {
                await this.stripeSubscriptionService.handleInvoicePaymentFailed(ctx, body, order);
            }
            core_1.Logger.info(`Successfully handled webhook ${body.type}`, constants_1.loggerCtx);
        }
        catch (error) {
            // Catch all for logging purposes
            core_1.Logger.error(`Failed to process incoming webhook ${body.type} (${body.id}): ${error?.message}`, constants_1.loggerCtx, error?.stack);
            throw error;
        }
    }
};
__decorate([
    (0, common_1.Post)('webhook'),
    __param(0, (0, common_1.Headers)('stripe-signature')),
    __param(1, (0, common_1.Req)()),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], StripeSubscriptionController.prototype, "webhook", null);
StripeSubscriptionController = __decorate([
    (0, common_1.Controller)('stripe-subscriptions'),
    __param(2, (0, common_1.Inject)(constants_1.PLUGIN_INIT_OPTIONS)),
    __metadata("design:paramtypes", [stripe_subscription_service_1.StripeSubscriptionService,
        core_1.OrderService, Object])
], StripeSubscriptionController);
exports.StripeSubscriptionController = StripeSubscriptionController;
