import { ActiveOrderService, ChannelService, CustomerService, EntityHydrator, EventBus, HistoryService, ID, JobQueueService, Order, OrderService, PaymentMethodService, ProductVariantService, RequestContext, SerializedRequestContext, TransactionalConnection } from '@vendure/core';
import { IncomingStripeWebhook } from './stripe.types';
import { OrderLineWithSubscriptionFields } from './subscription-custom-fields';
import { StripeClient } from './stripe.client';
import { StripeSubscriptionPricing, StripeSubscriptionPricingInput } from '../ui/generated/graphql';
import { Request } from 'express';
export interface StripeHandlerConfig {
    paymentMethodCode: string;
    stripeClient: StripeClient;
    webhookSecret: string;
}
interface CreateSubscriptionsJob {
    action: 'createSubscriptionsForOrder';
    ctx: SerializedRequestContext;
    orderCode: string;
    stripeCustomerId: string;
    stripePaymentMethodId: string;
}
interface CancelSubscriptionsJob {
    action: 'cancelSubscriptionsForOrderline';
    ctx: SerializedRequestContext;
    orderLineId: ID;
}
export type JobData = CreateSubscriptionsJob | CancelSubscriptionsJob;
export declare class StripeSubscriptionService {
    private paymentMethodService;
    private activeOrderService;
    private variantService;
    private entityHydrator;
    private channelService;
    private orderService;
    private historyService;
    private eventBus;
    private jobQueueService;
    private customerService;
    private connection;
    constructor(paymentMethodService: PaymentMethodService, activeOrderService: ActiveOrderService, variantService: ProductVariantService, entityHydrator: EntityHydrator, channelService: ChannelService, orderService: OrderService, historyService: HistoryService, eventBus: EventBus, jobQueueService: JobQueueService, customerService: CustomerService, connection: TransactionalConnection);
    private jobQueue;
    onModuleInit(): Promise<void>;
    cancelSubscriptionForOrderLine(ctx: RequestContext, orderLineId: ID): Promise<void>;
    createPaymentIntent(ctx: RequestContext): Promise<string>;
    /**
     * Used for previewing the prices including VAT of a subscription
     */
    getPricingForVariant(ctx: RequestContext, input: StripeSubscriptionPricingInput): Promise<StripeSubscriptionPricing>;
    /**
     *
     * Calculate subscription pricing based on an orderLine.
     * This differs from a variant, because orderLines can have discounts applied
     */
    getPricingForOrderLine(ctx: RequestContext, orderLine: OrderLineWithSubscriptionFields): Promise<StripeSubscriptionPricing>;
    /**
     * Handle future subscription payments that come in after the initial payment intent
     */
    handleInvoicePaymentSucceeded(ctx: RequestContext, { data: { object } }: IncomingStripeWebhook, order: Order): Promise<void>;
    /**
     * Handle failed subscription payments that come in after the initial payment intent
     */
    handleInvoicePaymentFailed(ctx: RequestContext, { data: { object } }: IncomingStripeWebhook, order: Order): Promise<void>;
    /**
     * Handle the initial payment Intent succeeded.
     * Creates subscriptions in Stripe for customer attached to this order
     */
    handlePaymentIntentSucceeded(ctx: RequestContext, { data: { object: eventData } }: IncomingStripeWebhook, order: Order): Promise<void>;
    /**
     * Create subscriptions for customer based on order
     */
    private createSubscriptions;
    saveSubscriptionIds(ctx: RequestContext, orderLineId: ID, subscriptionIds: string[]): Promise<void>;
    createContext(channelToken: string, req: Request): Promise<RequestContext>;
    /**
     * Get the paymentMethod with the stripe handler, should be only 1!
     */
    getStripeHandler(ctx: RequestContext, orderId: ID): Promise<StripeHandlerConfig>;
    logHistoryEntry(ctx: RequestContext, orderId: ID, message: string, error?: unknown, pricing?: StripeSubscriptionPricing, subscriptionId?: string): Promise<void>;
}
export {};
