"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StripeSubscriptionService = void 0;
const common_1 = require("@nestjs/common");
const generated_types_1 = require("@vendure/common/lib/generated-types");
const core_1 = require("@vendure/core");
const constants_1 = require("../constants");
const stripe_client_1 = require("./stripe.client");
const stripe_subscription_handler_1 = require("./stripe-subscription.handler");
const operators_1 = require("rxjs/operators");
const pricing_helper_1 = require("./pricing.helper");
const crypto_1 = require("crypto");
const has_stripe_subscription_products_payment_checker_1 = require("./has-stripe-subscription-products-payment-checker");
let StripeSubscriptionService = class StripeSubscriptionService {
    constructor(paymentMethodService, activeOrderService, variantService, entityHydrator, channelService, orderService, historyService, eventBus, jobQueueService, customerService, connection) {
        this.paymentMethodService = paymentMethodService;
        this.activeOrderService = activeOrderService;
        this.variantService = variantService;
        this.entityHydrator = entityHydrator;
        this.channelService = channelService;
        this.orderService = orderService;
        this.historyService = historyService;
        this.eventBus = eventBus;
        this.jobQueueService = jobQueueService;
        this.customerService = customerService;
        this.connection = connection;
    }
    async onModuleInit() {
        // Create jobQueue with handlers
        this.jobQueue = await this.jobQueueService.createQueue({
            name: 'stripe-subscription',
            process: async ({ data, id }) => {
                const ctx = core_1.RequestContext.deserialize(data.ctx);
                if (data.action === 'cancelSubscriptionsForOrderline') {
                    this.cancelSubscriptionForOrderLine(ctx, data.orderLineId);
                }
                else if (data.action === 'createSubscriptionsForOrder') {
                    const order = await this.orderService.findOneByCode(ctx, data.orderCode, []);
                    try {
                        await this.createSubscriptions(ctx, data.orderCode, data.stripeCustomerId, data.stripePaymentMethodId);
                    }
                    catch (error) {
                        core_1.Logger.warn(`Failed to process job ${data.action} (${id}) for channel ${data.ctx._channel.token}: ${error}`, constants_1.loggerCtx);
                        if (order) {
                            await this.logHistoryEntry(ctx, order.id, 'Failed to create subscription', error);
                        }
                        throw error;
                    }
                }
            },
        });
        // Add unique hash for subscriptions, so Vendure creates a new order line
        this.eventBus.ofType(core_1.OrderLineEvent).subscribe(async (event) => {
            const orderLine = event.orderLine;
            if (event.type === 'created' &&
                orderLine.productVariant.customFields.subscriptionSchedule) {
                await this.connection
                    .getRepository(event.ctx, core_1.OrderLine)
                    .update({ id: event.orderLine.id }, { customFields: { subscriptionHash: (0, crypto_1.randomUUID)() } });
            }
        });
        // Listen for stock cancellation or release events
        this.eventBus
            .ofType(core_1.StockMovementEvent)
            .pipe(
        // Filter by event type
        (0, operators_1.filter)((event) => event.type === generated_types_1.StockMovementType.RELEASE ||
            event.type === generated_types_1.StockMovementType.CANCELLATION))
            .subscribe(async (event) => {
            const orderLinesWithSubscriptions = event.stockMovements
                .map((stockMovement) => stockMovement.orderLine)
                // Filter out non-sub orderlines
                .filter((orderLine) => orderLine.customFields.subscriptionIds);
            await Promise.all(
            // Push jobs
            orderLinesWithSubscriptions.map((line) => this.jobQueue.add({
                ctx: event.ctx.serialize(),
                action: 'cancelSubscriptionsForOrderline',
                orderLineId: line.id,
            })));
        });
    }
    async cancelSubscriptionForOrderLine(ctx, orderLineId) {
        const order = (await this.orderService.findOneByOrderLineId(ctx, orderLineId, ['lines']));
        if (!order) {
            throw Error(`Order for OrderLine ${orderLineId} not found`);
        }
        const line = order?.lines.find((l) => l.id == orderLineId);
        if (!line?.customFields.subscriptionIds?.length) {
            return core_1.Logger.info(`OrderLine ${orderLineId} of ${orderLineId} has no subscriptionIds. Not cancelling anything... `, constants_1.loggerCtx);
        }
        await this.entityHydrator.hydrate(ctx, line, { relations: ['order'] });
        const { stripeClient } = await this.getStripeHandler(ctx, line.order.id);
        for (const subscriptionId of line.customFields.subscriptionIds) {
            try {
                await stripeClient.subscriptions.update(subscriptionId, {
                    cancel_at_period_end: true,
                });
                core_1.Logger.info(`Cancelled subscription ${subscriptionId}`);
                await this.logHistoryEntry(ctx, order.id, `Cancelled subscription ${subscriptionId}`, undefined, undefined, subscriptionId);
            }
            catch (e) {
                core_1.Logger.error(`Failed to cancel subscription ${subscriptionId}`, constants_1.loggerCtx);
                await this.logHistoryEntry(ctx, order.id, `Failed to cancel ${subscriptionId}`, e, undefined, subscriptionId);
            }
        }
    }
    async createPaymentIntent(ctx) {
        let order = (await this.activeOrderService.getActiveOrder(ctx, undefined));
        if (!order) {
            throw new core_1.UserInputError('No active order for session');
        }
        if (!order.totalWithTax) {
            // Add a verification fee to the order to support orders that are actually $0
            order = (await this.orderService.addSurchargeToOrder(ctx, order.id, {
                description: 'Verification fee',
                listPrice: 100,
                listPriceIncludesTax: true,
            }));
        }
        await this.entityHydrator.hydrate(ctx, order, {
            relations: ['customer', 'shippingLines', 'lines.productVariant'],
        });
        if (!order.lines?.length) {
            throw new core_1.UserInputError('Cannot create payment intent for empty order');
        }
        if (!order.customer) {
            throw new core_1.UserInputError('Cannot create payment intent for order without customer');
        }
        if (!order.shippingLines?.length) {
            throw new core_1.UserInputError('Cannot create payment intent for order without shippingMethod');
        }
        const { stripeClient } = await this.getStripeHandler(ctx, order.id);
        const stripeCustomer = await stripeClient.getOrCreateClient(order.customer);
        this.customerService
            .update(ctx, {
            id: order.customer.id,
            customFields: {
                stripeSubscriptionCustomerId: stripeCustomer.id,
            },
        })
            .catch((err) => core_1.Logger.error(`Failed to update stripeCustomerId ${stripeCustomer.id} for ${order.customer.emailAddress}`, constants_1.loggerCtx, err));
        const hasSubscriptionProducts = order.lines.some((l) => l.productVariant.customFields.subscriptionSchedule);
        const intent = await stripeClient.paymentIntents.create({
            customer: stripeCustomer.id,
            payment_method_types: ['card'],
            setup_future_usage: hasSubscriptionProducts
                ? 'off_session'
                : 'on_session',
            amount: order.totalWithTax,
            currency: order.currencyCode,
            metadata: {
                orderCode: order.code,
                channelToken: ctx.channel.token,
                amount: order.totalWithTax,
            },
        });
        core_1.Logger.info(`Created payment intent '${intent.id}' for order ${order.code}`, constants_1.loggerCtx);
        return intent.client_secret;
    }
    /**
     * Used for previewing the prices including VAT of a subscription
     */
    async getPricingForVariant(ctx, input) {
        const variant = (await this.variantService.findOne(ctx, input.productVariantId));
        if (!variant || !variant?.enabled) {
            throw new core_1.UserInputError(`No variant found with id ${input.productVariantId}`);
        }
        if (!variant.listPrice) {
            throw new core_1.UserInputError(`Variant ${variant.id} doesn't have a "listPrice". Variant.listPrice is needed to calculate subscription pricing`);
        }
        if (!variant.customFields.subscriptionSchedule) {
            throw new core_1.UserInputError(`Variant ${variant.id} doesn't have a schedule attached`);
        }
        const subscriptionPricing = (0, pricing_helper_1.calculateSubscriptionPricing)(ctx, variant.listPrice, variant.customFields.subscriptionSchedule, input);
        return {
            ...subscriptionPricing,
            variantId: variant.id,
            // original price is the same as the recurring price without discounts
            originalRecurringPrice: subscriptionPricing.recurringPrice,
        };
    }
    /**
     *
     * Calculate subscription pricing based on an orderLine.
     * This differs from a variant, because orderLines can have discounts applied
     */
    async getPricingForOrderLine(ctx, orderLine) {
        await this.entityHydrator.hydrate(ctx, orderLine, {
            relations: ['productVariant.taxCategory', 'order', 'order.promotions'],
            applyProductVariantPrices: true,
        });
        if (!orderLine.productVariant?.enabled) {
            throw new core_1.UserInputError(`Variant ${orderLine.productVariant.sku} is not enabled`);
        }
        if (!orderLine.productVariant.customFields.subscriptionSchedule) {
            throw new core_1.UserInputError(`Variant ${orderLine.productVariant.id} doesn't have a schedule attached`);
        }
        const subscriptionPricing = (0, pricing_helper_1.calculateSubscriptionPricing)(ctx, orderLine.productVariant.listPrice, orderLine.productVariant.customFields.subscriptionSchedule, {
            downpayment: orderLine.customFields.downpayment,
            startDate: orderLine.customFields.startDate,
        });
        // Execute promotions on recurringPrice
        const discountedRecurringPrice = await (0, pricing_helper_1.applySubscriptionPromotions)(ctx, subscriptionPricing.recurringPrice, orderLine, orderLine.order.promotions || []);
        return {
            ...subscriptionPricing,
            variantId: orderLine.productVariant.id,
            originalRecurringPrice: subscriptionPricing.recurringPrice,
            recurringPrice: Math.round(discountedRecurringPrice),
        };
    }
    /**
     * Handle future subscription payments that come in after the initial payment intent
     */
    async handleInvoicePaymentSucceeded(ctx, { data: { object } }, order) {
        const amount = object.lines?.data?.[0]?.plan?.amount;
        const message = amount
            ? `Received subscription payment of ${(0, pricing_helper_1.printMoney)(amount)}`
            : 'Received subscription payment';
        await this.logHistoryEntry(ctx, order.id, message, undefined, undefined, object.subscription);
    }
    /**
     * Handle failed subscription payments that come in after the initial payment intent
     */
    async handleInvoicePaymentFailed(ctx, { data: { object } }, order) {
        const amount = object.lines?.data[0]?.plan?.amount;
        const message = amount
            ? `Subscription payment of ${(0, pricing_helper_1.printMoney)(amount)} failed`
            : 'Subscription payment failed';
        await this.logHistoryEntry(ctx, order.id, message, `${message} - ${object.id}`, undefined, object.subscription);
    }
    /**
     * Handle the initial payment Intent succeeded.
     * Creates subscriptions in Stripe for customer attached to this order
     */
    async handlePaymentIntentSucceeded(ctx, { data: { object: eventData } }, order) {
        const { paymentMethodCode } = await this.getStripeHandler(ctx, order.id);
        if (!eventData.customer) {
            await this.logHistoryEntry(ctx, order.id, '', `No customer ID found in incoming webhook. Can not create subscriptions for this order.`);
            throw Error(`No customer found in webhook data for order ${order.code}`);
        }
        // Create subscriptions for customer
        this.jobQueue
            .add({
            action: 'createSubscriptionsForOrder',
            ctx: ctx.serialize(),
            orderCode: order.code,
            stripePaymentMethodId: eventData.payment_method,
            stripeCustomerId: eventData.customer,
        }, { retries: 0 } // Only 1 try, because subscription creation isn't transaction-proof
        )
            .catch((e) => core_1.Logger.error(`Failed to add subscription-creation job to queue`, constants_1.loggerCtx));
        // Status is complete, we can settle payment
        if (order.state !== 'ArrangingPayment') {
            const transitionToStateResult = await this.orderService.transitionToState(ctx, order.id, 'ArrangingPayment');
            if (transitionToStateResult instanceof core_1.OrderStateTransitionError) {
                throw Error(`Error transitioning order ${order.code} from ${transitionToStateResult.fromState} to ${transitionToStateResult.toState}: ${transitionToStateResult.message}`);
            }
        }
        const addPaymentToOrderResult = await this.orderService.addPaymentToOrder(ctx, order.id, {
            method: paymentMethodCode,
            metadata: {
                setupIntentId: eventData.id,
                amount: eventData.metadata.amount,
            },
        });
        if (addPaymentToOrderResult.errorCode) {
            throw Error(`Error adding payment to order ${order.code}: ${addPaymentToOrderResult.message}`);
        }
        core_1.Logger.info(`Successfully settled payment for order ${order.code} for channel ${ctx.channel.token}`, constants_1.loggerCtx);
    }
    /**
     * Create subscriptions for customer based on order
     */
    async createSubscriptions(ctx, orderCode, stripeCustomerId, stripePaymentMethodId) {
        const order = (await this.orderService.findOneByCode(ctx, orderCode, [
            'customer',
            'lines',
            'lines.productVariant',
        ]));
        if (!order) {
            throw Error(`Cannot find order with code ${orderCode}`);
        }
        if (!(0, has_stripe_subscription_products_payment_checker_1.hasSubscriptions)(order)) {
            return core_1.Logger.info(`Not creating subscriptions for order ${order.code}, because it doesn't have any subscription products`);
        }
        const { stripeClient } = await this.getStripeHandler(ctx, order.id);
        const customer = await stripeClient.customers.retrieve(stripeCustomerId);
        if (!customer) {
            throw Error(`Failed to create subscription for ${stripeCustomerId} because the customer doesn't exist in Stripe`);
        }
        let orderLineCount = 0;
        const subscriptionOrderLines = order.lines.filter((line) => line.productVariant.customFields.subscriptionSchedule);
        for (const orderLine of subscriptionOrderLines) {
            orderLineCount++; // Start with 1
            const createdSubscriptions = [];
            const pricing = await this.getPricingForOrderLine(ctx, orderLine);
            if (pricing.schedule.paidUpFront && !pricing.schedule.autoRenew) {
                continue; // Paid up front without autoRenew doesn't need a subscription
            }
            core_1.Logger.info(`Creating subscriptions for ${orderCode}`, constants_1.loggerCtx);
            try {
                const product = await stripeClient.products.create({
                    name: `${orderLine.productVariant.name} (${order.code})`,
                });
                const recurringSubscription = await stripeClient.createOffSessionSubscription({
                    customerId: stripeCustomerId,
                    productId: product.id,
                    currencyCode: order.currencyCode,
                    amount: pricing.recurringPrice,
                    interval: pricing.interval,
                    intervalCount: pricing.intervalCount,
                    paymentMethodId: stripePaymentMethodId,
                    startDate: pricing.subscriptionStartDate,
                    endDate: pricing.subscriptionEndDate || undefined,
                    description: orderLine.productVariant.name,
                    orderCode: order.code,
                    channelToken: ctx.channel.token,
                });
                createdSubscriptions.push(recurringSubscription.id);
                if (recurringSubscription.status !== 'active' &&
                    recurringSubscription.status !== 'trialing') {
                    core_1.Logger.error(`Failed to create active subscription ${recurringSubscription.id} for order ${order.code}! It is still in status '${recurringSubscription.status}'`, constants_1.loggerCtx);
                    await this.logHistoryEntry(ctx, order.id, 'Failed to create subscription', `Subscription status is ${recurringSubscription.status}`, pricing, recurringSubscription.id);
                }
                else {
                    core_1.Logger.info(`Created subscription ${recurringSubscription.id}: ${(0, pricing_helper_1.printMoney)(pricing.recurringPrice)} every ${pricing.intervalCount} ${pricing.interval}(s) with startDate ${pricing.subscriptionStartDate} for order ${order.code}`, constants_1.loggerCtx);
                    await this.logHistoryEntry(ctx, order.id, `Created subscription for line ${orderLineCount}`, undefined, pricing, recurringSubscription.id);
                }
                if (pricing.downpayment) {
                    // Create downpayment with the interval of the duration. So, if the subscription renews in 6 months, then the downpayment should occur every 6 months
                    const downpaymentProduct = await stripeClient.products.create({
                        name: `${orderLine.productVariant.name} - Downpayment (${order.code})`,
                    });
                    const schedule = orderLine.productVariant.customFields.subscriptionSchedule;
                    if (!schedule) {
                        throw new core_1.UserInputError(`Variant ${orderLine.productVariant.id} doesn't have a schedule attached`);
                    }
                    const downpaymentInterval = schedule.durationInterval;
                    const downpaymentIntervalCount = schedule.durationCount;
                    const nextDownpaymentDate = (0, pricing_helper_1.getNextCyclesStartDate)(new Date(), schedule.startMoment, schedule.durationInterval, schedule.durationCount, schedule.fixedStartDate);
                    const downpaymentSubscription = await stripeClient.createOffSessionSubscription({
                        customerId: stripeCustomerId,
                        productId: downpaymentProduct.id,
                        currencyCode: order.currencyCode,
                        amount: pricing.downpayment,
                        interval: downpaymentInterval,
                        intervalCount: downpaymentIntervalCount,
                        paymentMethodId: stripePaymentMethodId,
                        startDate: nextDownpaymentDate,
                        endDate: pricing.subscriptionEndDate || undefined,
                        description: `Downpayment`,
                        orderCode: order.code,
                        channelToken: ctx.channel.token,
                    });
                    createdSubscriptions.push(recurringSubscription.id);
                    if (downpaymentSubscription.status !== 'active' &&
                        downpaymentSubscription.status !== 'trialing') {
                        core_1.Logger.error(`Failed to create active subscription ${downpaymentSubscription.id} for order ${order.code}! It is still in status '${downpaymentSubscription.status}'`, constants_1.loggerCtx);
                        await this.logHistoryEntry(ctx, order.id, 'Failed to create downpayment subscription', 'Failed to create active subscription', undefined, downpaymentSubscription.id);
                    }
                    else {
                        core_1.Logger.info(`Created downpayment subscription ${downpaymentSubscription.id}: ${(0, pricing_helper_1.printMoney)(pricing.downpayment)} every ${downpaymentIntervalCount} ${downpaymentInterval}(s) with startDate ${pricing.subscriptionStartDate} for order ${order.code}`, constants_1.loggerCtx);
                        await this.logHistoryEntry(ctx, order.id, `Created downpayment subscription for line ${orderLineCount}`, undefined, pricing, downpaymentSubscription.id);
                    }
                }
                await this.saveSubscriptionIds(ctx, orderLine.id, createdSubscriptions);
            }
            catch (e) {
                await this.logHistoryEntry(ctx, order.id, '', e);
                throw e;
            }
        }
    }
    async saveSubscriptionIds(ctx, orderLineId, subscriptionIds) {
        await this.connection
            .getRepository(ctx, core_1.OrderLine)
            .update({ id: orderLineId }, { customFields: { subscriptionIds } });
    }
    async createContext(channelToken, req) {
        const channel = await this.channelService.getChannelFromToken(channelToken);
        return new core_1.RequestContext({
            apiType: 'admin',
            isAuthorized: true,
            authorizedAsOwnerOnly: false,
            channel,
            languageCode: core_1.LanguageCode.en,
            req,
        });
    }
    /**
     * Get the paymentMethod with the stripe handler, should be only 1!
     */
    async getStripeHandler(ctx, orderId) {
        const paymentMethodQuotes = await this.orderService.getEligiblePaymentMethods(ctx, orderId);
        const paymentMethodQuote = paymentMethodQuotes
            .filter((quote) => quote.isEligible)
            .find((pm) => pm.code.indexOf('stripe-subscription') > -1);
        if (!paymentMethodQuote) {
            throw new core_1.UserInputError(`No eligible payment method found with code 'stripe-subscription'`);
        }
        const paymentMethod = await this.paymentMethodService.findOne(ctx, paymentMethodQuote.id);
        if (!paymentMethod ||
            paymentMethod.handler.code !== stripe_subscription_handler_1.stripeSubscriptionHandler.code) {
            throw new core_1.UserInputError(`Payment method '${paymentMethodQuote.code}' doesn't have handler '${stripe_subscription_handler_1.stripeSubscriptionHandler.code}' configured.`);
        }
        const apiKey = paymentMethod.handler.args.find((arg) => arg.name === 'apiKey')?.value;
        let webhookSecret = paymentMethod.handler.args.find((arg) => arg.name === 'webhookSecret')?.value;
        if (!apiKey || !webhookSecret) {
            core_1.Logger.warn(`No api key or webhook secret is configured for ${paymentMethod.code}`, constants_1.loggerCtx);
            throw Error(`Payment method ${paymentMethod.code} has no api key or webhook secret configured`);
        }
        return {
            paymentMethodCode: paymentMethod.code,
            stripeClient: new stripe_client_1.StripeClient(webhookSecret, apiKey, {
                apiVersion: null, // Null uses accounts default version
            }),
            webhookSecret,
        };
    }
    async logHistoryEntry(ctx, orderId, message, error, pricing, subscriptionId) {
        let prettifiedError = error
            ? JSON.parse(JSON.stringify(error, Object.getOwnPropertyNames(error)))
            : undefined; // Make sure its serializable
        let prettifierPricing = pricing
            ? {
                ...pricing,
                totalProratedAmount: (0, pricing_helper_1.printMoney)(pricing.totalProratedAmount),
                downpayment: (0, pricing_helper_1.printMoney)(pricing.downpayment),
                recurringPrice: (0, pricing_helper_1.printMoney)(pricing.recurringPrice),
                amountDueNow: (0, pricing_helper_1.printMoney)(pricing.amountDueNow),
                dayRate: (0, pricing_helper_1.printMoney)(pricing.dayRate),
            }
            : undefined;
        await this.historyService.createHistoryEntryForOrder({
            ctx,
            orderId,
            type: 'STRIPE_SUBSCRIPTION_NOTIFICATION',
            data: {
                message,
                valid: !error,
                error: prettifiedError,
                subscriptionId,
                pricing: prettifierPricing,
            },
        }, false);
    }
};
StripeSubscriptionService = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [core_1.PaymentMethodService,
        core_1.ActiveOrderService,
        core_1.ProductVariantService,
        core_1.EntityHydrator,
        core_1.ChannelService,
        core_1.OrderService,
        core_1.HistoryService,
        core_1.EventBus,
        core_1.JobQueueService,
        core_1.CustomerService,
        core_1.TransactionalConnection])
], StripeSubscriptionService);
exports.StripeSubscriptionService = StripeSubscriptionService;
