/// <reference types="node" />
import Stripe from 'stripe';
import { CustomerWithSubscriptionFields } from './subscription-custom-fields';
interface SubscriptionInput {
    customerId: string;
    productId: string;
    currencyCode: string;
    amount: number;
    interval: Stripe.SubscriptionCreateParams.Item.PriceData.Recurring.Interval;
    intervalCount: number;
    paymentMethodId: string;
    startDate: Date;
    orderCode: string;
    channelToken: string;
    endDate?: Date;
    description?: string;
}
/**
 * Wrapper around the Stripe client with specifics for this subscription plugin
 */
export declare class StripeClient extends Stripe {
    private webhookSecret;
    constructor(webhookSecret: string, apiKey: string, config: Stripe.StripeConfig);
    getOrCreateClient(customer: CustomerWithSubscriptionFields): Promise<Stripe.Customer>;
    /**
     * Throws an error if incoming webhook signature is invalid
     */
    validateWebhookSignature(payload: Buffer, signature: string | undefined): void;
    createOffSessionSubscription({ customerId, productId, currencyCode, amount, interval, intervalCount, paymentMethodId, startDate, endDate, description, orderCode, channelToken, }: SubscriptionInput): Promise<Stripe.Subscription>;
    toStripeTimeStamp(date: Date): number;
}
export {};
