import { Customer, CustomFieldConfig, Order, OrderLine, ProductVariant } from '@vendure/core';
import { Schedule } from './schedule.entity';
import { StripeSubscriptionPricing } from '../ui/generated/graphql';
/**
 * Custom fields for managing subscriptions.
 * See {@link productVariantCustomFields} for more information on each field
 */
export interface VariantWithSubscriptionFields extends ProductVariant {
    customFields: {
        subscriptionSchedule?: Schedule;
    };
}
export interface CustomerWithSubscriptionFields extends Customer {
    customFields: {
        stripeSubscriptionCustomerId?: string;
    };
}
export interface OrderLineWithSubscriptionFields extends OrderLine {
    subscriptionPricing?: StripeSubscriptionPricing;
    customFields: {
        downpayment?: number;
        startDate?: Date;
        subscriptionIds?: string[];
        /**
         * Unique hash to separate order lines
         */
        subscriptionHash?: string;
    };
    productVariant: VariantWithSubscriptionFields;
}
/**
 * An order that can have subscriptions in it
 */
export interface OrderWithSubscriptionFields extends Order {
    lines: (OrderLineWithSubscriptionFields & {
        productVariant: VariantWithSubscriptionFields;
    })[];
    customer: CustomerWithSubscriptionFields;
}
export declare const productVariantCustomFields: CustomFieldConfig[];
export declare const customerCustomFields: CustomFieldConfig[];
export declare const orderLineCustomFields: CustomFieldConfig[];
