import { ConfigArg } from '@vendure/common/lib/generated-types';
import { LanguageCode, PromotionCondition, PromotionOrderAction, PromotionOrderActionConfig, RequestContext } from '@vendure/core';
import { ConfigArgValues, ConfigArgs } from '@vendure/core/dist/common/configurable-operation';
import { OrderLineWithSubscriptionFields } from './subscription-custom-fields';
/**
 * Function that executes on the given subscription
 */
type ExecuteOnSubscriptionFn<T extends ConfigArgs> = (ctx: RequestContext, currentSubscriptionPrice: number, orderLine: OrderLineWithSubscriptionFields, args: ConfigArgValues<T>) => Promise<number>;
/**
 * A custom Promotion Action that can discount specific subscription items in an order.
 */
export declare class SubscriptionPromotionAction<U extends Array<PromotionCondition<any>>, T extends ConfigArgs = ConfigArgs> extends PromotionOrderAction<T, U> {
    executeOnSubscriptionFn: ExecuteOnSubscriptionFn<T>;
    constructor(config: Omit<PromotionOrderActionConfig<T, U>, 'execute'> & {
        executeOnSubscription: ExecuteOnSubscriptionFn<T>;
    });
    executeOnSubscription(ctx: RequestContext, currentSubscriptionPrice: number, orderLine: OrderLineWithSubscriptionFields, args: ConfigArg[]): Promise<number>;
}
export declare const allByPercentage: SubscriptionPromotionAction<PromotionCondition<any, string, any>[], {
    discount: {
        type: "float";
        ui: {
            component: string;
            suffix: string;
        };
    };
}>;
export declare const withFacetsByPercentage: SubscriptionPromotionAction<PromotionCondition<any, string, any>[], {
    discount: {
        type: "float";
        ui: {
            component: string;
            suffix: string;
        };
    };
    facets: {
        type: "ID";
        list: true;
        ui: {
            component: string;
        };
    };
}>;
export declare const withFacetsByFixedAmount: SubscriptionPromotionAction<PromotionCondition<any, string, any>[], {
    amount: {
        type: "int";
        ui: {
            component: string;
        };
    };
    facets: {
        type: "ID";
        list: true;
        ui: {
            component: string;
        };
    };
}>;
export declare const selectedProductsByPercentage: SubscriptionPromotionAction<PromotionCondition<any, string, any>[], {
    discount: {
        type: "float";
        ui: {
            component: string;
            suffix: string;
        };
    };
    productVariantIds: {
        type: "ID";
        list: true;
        ui: {
            component: string;
        };
        label: {
            languageCode: LanguageCode.en;
            value: string;
        }[];
    };
}>;
export declare const selectedProductsByFixedAmount: SubscriptionPromotionAction<PromotionCondition<any, string, any>[], {
    amount: {
        type: "int";
        ui: {
            component: string;
        };
    };
    productVariantIds: {
        type: "ID";
        list: true;
        ui: {
            component: string;
        };
        label: {
            languageCode: LanguageCode.en;
            value: string;
        }[];
    };
}>;
export declare const subscriptionPromotions: (SubscriptionPromotionAction<PromotionCondition<any, string, any>[], {
    discount: {
        type: "float";
        ui: {
            component: string;
            suffix: string;
        };
    };
}> | SubscriptionPromotionAction<PromotionCondition<any, string, any>[], {
    discount: {
        type: "float";
        ui: {
            component: string;
            suffix: string;
        };
    };
    facets: {
        type: "ID";
        list: true;
        ui: {
            component: string;
        };
    };
}> | SubscriptionPromotionAction<PromotionCondition<any, string, any>[], {
    amount: {
        type: "int";
        ui: {
            component: string;
        };
    };
    facets: {
        type: "ID";
        list: true;
        ui: {
            component: string;
        };
    };
}> | SubscriptionPromotionAction<PromotionCondition<any, string, any>[], {
    discount: {
        type: "float";
        ui: {
            component: string;
            suffix: string;
        };
    };
    productVariantIds: {
        type: "ID";
        list: true;
        ui: {
            component: string;
        };
        label: {
            languageCode: LanguageCode.en;
            value: string;
        }[];
    };
}> | SubscriptionPromotionAction<PromotionCondition<any, string, any>[], {
    amount: {
        type: "int";
        ui: {
            component: string;
        };
    };
    productVariantIds: {
        type: "ID";
        list: true;
        ui: {
            component: string;
        };
        label: {
            languageCode: LanguageCode.en;
            value: string;
        }[];
    };
}>)[];
export {};
