"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var StripeSubscriptionPlugin_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.StripeSubscriptionPlugin = void 0;
const core_1 = require("@vendure/core");
const constants_1 = require("./constants");
const subscription_custom_fields_1 = require("./api/subscription-custom-fields");
const raw_body_1 = require("../../util/src/raw-body");
const subscription_order_item_calculation_1 = require("./api/subscription-order-item-calculation");
const schedule_entity_1 = require("./api/schedule.entity");
const schedule_service_1 = require("./api/schedule.service");
const path_1 = __importDefault(require("path"));
const graphql_schemas_1 = require("./api/graphql-schemas");
const stripe_subscription_controller_1 = require("./api/stripe-subscription.controller");
const stripe_subscription_service_1 = require("./api/stripe-subscription.service");
const stripe_subscription_handler_1 = require("./api/stripe-subscription.handler");
const has_stripe_subscription_products_payment_checker_1 = require("./api/has-stripe-subscription-products-payment-checker");
const subscription_promotion_1 = require("./api/subscription.promotion");
let StripeSubscriptionPlugin = StripeSubscriptionPlugin_1 = class StripeSubscriptionPlugin {
    static init(options) {
        this.options = options;
        return StripeSubscriptionPlugin_1;
    }
};
StripeSubscriptionPlugin.ui = {
    extensionPath: path_1.default.join(__dirname, 'ui'),
    ngModules: [
        {
            type: 'lazy',
            route: 'subscription-schedules',
            ngModuleFileName: 'schedules.module.ts',
            ngModuleName: 'SchedulesModule',
        },
        {
            type: 'shared',
            ngModuleFileName: 'stripe-subscription-shared.module.ts',
            ngModuleName: 'StripeSubscriptionSharedModule',
        },
    ],
};
StripeSubscriptionPlugin = StripeSubscriptionPlugin_1 = __decorate([
    (0, core_1.VendurePlugin)({
        imports: [core_1.PluginCommonModule],
        entities: [schedule_entity_1.Schedule],
        shopApiExtensions: {
            schema: graphql_schemas_1.shopSchemaExtensions,
            resolvers: [stripe_subscription_controller_1.ShopResolver, stripe_subscription_controller_1.ShopOrderLinePricingResolver],
        },
        adminApiExtensions: {
            schema: graphql_schemas_1.adminSchemaExtensions,
            resolvers: [stripe_subscription_controller_1.AdminResolver, stripe_subscription_controller_1.AdminPriceIncludesTaxResolver],
        },
        controllers: [stripe_subscription_controller_1.StripeSubscriptionController],
        providers: [
            stripe_subscription_service_1.StripeSubscriptionService,
            schedule_service_1.ScheduleService,
            {
                provide: constants_1.PLUGIN_INIT_OPTIONS,
                useFactory: () => StripeSubscriptionPlugin_1.options,
            },
        ],
        configuration: (config) => {
            config.paymentOptions.paymentMethodHandlers.push(stripe_subscription_handler_1.stripeSubscriptionHandler);
            config.paymentOptions.paymentMethodEligibilityCheckers = [
                ...(config.paymentOptions.paymentMethodEligibilityCheckers ?? []),
                has_stripe_subscription_products_payment_checker_1.hasStripeSubscriptionProductsPaymentChecker,
            ];
            config.apiOptions.middleware.push((0, raw_body_1.createRawBodyMiddleWare)('/stripe-subscription*'));
            config.orderOptions.orderItemPriceCalculationStrategy =
                new subscription_order_item_calculation_1.SubscriptionOrderItemCalculation();
            config.customFields.ProductVariant.push(...subscription_custom_fields_1.productVariantCustomFields);
            config.customFields.Customer.push(...subscription_custom_fields_1.customerCustomFields);
            config.customFields.OrderLine.push(...subscription_custom_fields_1.orderLineCustomFields);
            config.promotionOptions.promotionActions.push(...subscription_promotion_1.subscriptionPromotions);
            return config;
        },
    })
], StripeSubscriptionPlugin);
exports.StripeSubscriptionPlugin = StripeSubscriptionPlugin;
