export type Maybe<T> = T | null;
export type InputMaybe<T> = Maybe<T>;
export type Exact<T extends { [key: string]: unknown }> = { [K in keyof T]: T[K] };
export type MakeOptional<T, K extends keyof T> = Omit<T, K> & { [SubKey in K]?: Maybe<T[SubKey]> };
export type MakeMaybe<T, K extends keyof T> = Omit<T, K> & { [SubKey in K]: Maybe<T[SubKey]> };
/** All built-in and custom scalars, mapped to their actual values */
export type Scalars = {
  ID: string | number;
  String: string;
  Boolean: boolean;
  Int: number;
  Float: number;
  DateTime: Date;
};

export enum HistoryEntryType {
  StripeSubscriptionNotification = 'STRIPE_SUBSCRIPTION_NOTIFICATION'
}

export type Mutation = {
  __typename?: 'Mutation';
  createStripeSubscriptionIntent: Scalars['String'];
  deleteStripeSubscriptionSchedule?: Maybe<Scalars['Boolean']>;
  upsertStripeSubscriptionSchedule: StripeSubscriptionSchedule;
};


export type MutationDeleteStripeSubscriptionScheduleArgs = {
  scheduleId: Scalars['ID'];
};


export type MutationUpsertStripeSubscriptionScheduleArgs = {
  input: UpsertStripeSubscriptionScheduleInput;
};

export type OrderLine = {
  __typename?: 'OrderLine';
  subscriptionPricing?: Maybe<StripeSubscriptionPricing>;
};

export type Query = {
  __typename?: 'Query';
  /**
   * Preview the pricing model of a given subscription.
   * Start date and downpayment are optional: if not supplied, the subscriptions default will be used.
   */
  stripeSubscriptionPricing?: Maybe<StripeSubscriptionPricing>;
  stripeSubscriptionPricingForProduct: Array<StripeSubscriptionPricing>;
  stripeSubscriptionSchedules: Array<StripeSubscriptionSchedule>;
};


export type QueryStripeSubscriptionPricingArgs = {
  input?: InputMaybe<StripeSubscriptionPricingInput>;
};


export type QueryStripeSubscriptionPricingForProductArgs = {
  productId: Scalars['ID'];
};

export type StripeSubscriptionPricing = {
  __typename?: 'StripeSubscriptionPricing';
  amountDueNow: Scalars['Int'];
  dayRate: Scalars['Int'];
  downpayment: Scalars['Int'];
  interval: SubscriptionInterval;
  intervalCount: Scalars['Int'];
  /** The original recurring price of the subscription, including tax, without discounts applied. */
  originalRecurringPrice: Scalars['Int'];
  pricesIncludeTax: Scalars['Boolean'];
  proratedDays: Scalars['Int'];
  /** The recurring price of the subscription, including discounts and tax. */
  recurringPrice: Scalars['Int'];
  schedule: StripeSubscriptionSchedule;
  subscriptionEndDate?: Maybe<Scalars['DateTime']>;
  subscriptionStartDate: Scalars['DateTime'];
  totalProratedAmount: Scalars['Int'];
  variantId: Scalars['String'];
};

export type StripeSubscriptionPricingInput = {
  downpayment?: InputMaybe<Scalars['Int']>;
  productVariantId: Scalars['ID'];
  startDate?: InputMaybe<Scalars['DateTime']>;
};

export type StripeSubscriptionSchedule = {
  __typename?: 'StripeSubscriptionSchedule';
  autoRenew?: Maybe<Scalars['Boolean']>;
  billingCount: Scalars['Int'];
  billingInterval: SubscriptionInterval;
  createdAt?: Maybe<Scalars['DateTime']>;
  downpayment: Scalars['Int'];
  durationCount: Scalars['Int'];
  durationInterval: SubscriptionInterval;
  fixedStartDate?: Maybe<Scalars['DateTime']>;
  id: Scalars['ID'];
  name: Scalars['String'];
  paidUpFront: Scalars['Boolean'];
  pricesIncludeTax: Scalars['Boolean'];
  startMoment: SubscriptionStartMoment;
  updatedAt?: Maybe<Scalars['DateTime']>;
  useProration?: Maybe<Scalars['Boolean']>;
};

export enum SubscriptionInterval {
  Month = 'month',
  Week = 'week'
}

export enum SubscriptionStartMoment {
  EndOfBillingInterval = 'end_of_billing_interval',
  FixedStartdate = 'fixed_startdate',
  StartOfBillingInterval = 'start_of_billing_interval',
  TimeOfPurchase = 'time_of_purchase'
}

export type UpsertStripeSubscriptionScheduleInput = {
  autoRenew?: InputMaybe<Scalars['Boolean']>;
  billingCount: Scalars['Int'];
  billingInterval: SubscriptionInterval;
  downpayment: Scalars['Int'];
  durationCount: Scalars['Int'];
  durationInterval: SubscriptionInterval;
  fixedStartDate?: InputMaybe<Scalars['DateTime']>;
  id?: InputMaybe<Scalars['ID']>;
  name: Scalars['String'];
  startMoment: SubscriptionStartMoment;
  useProration?: InputMaybe<Scalars['Boolean']>;
};

export type ScheduleFieldsFragment = { __typename?: 'StripeSubscriptionSchedule', id: string | number, createdAt?: Date | null, updatedAt?: Date | null, name: string, downpayment: number, durationInterval: SubscriptionInterval, durationCount: number, startMoment: SubscriptionStartMoment, billingInterval: SubscriptionInterval, billingCount: number, paidUpFront: boolean, fixedStartDate?: Date | null, useProration?: boolean | null, autoRenew?: boolean | null };

export type StripeSubscriptionSchedulesQueryVariables = Exact<{ [key: string]: never; }>;


export type StripeSubscriptionSchedulesQuery = { __typename?: 'Query', stripeSubscriptionSchedules: Array<{ __typename?: 'StripeSubscriptionSchedule', id: string | number, createdAt?: Date | null, updatedAt?: Date | null, name: string, downpayment: number, durationInterval: SubscriptionInterval, durationCount: number, startMoment: SubscriptionStartMoment, billingInterval: SubscriptionInterval, billingCount: number, paidUpFront: boolean, fixedStartDate?: Date | null, useProration?: boolean | null, autoRenew?: boolean | null }> };

export type UpsertStripeSubscriptionScheduleMutationVariables = Exact<{
  input: UpsertStripeSubscriptionScheduleInput;
}>;


export type UpsertStripeSubscriptionScheduleMutation = { __typename?: 'Mutation', upsertStripeSubscriptionSchedule: { __typename?: 'StripeSubscriptionSchedule', id: string | number, createdAt?: Date | null, updatedAt?: Date | null, name: string, downpayment: number, durationInterval: SubscriptionInterval, durationCount: number, startMoment: SubscriptionStartMoment, billingInterval: SubscriptionInterval, billingCount: number, paidUpFront: boolean, fixedStartDate?: Date | null, useProration?: boolean | null, autoRenew?: boolean | null } };

export type DeleteStripeSubscriptionScheduleMutationVariables = Exact<{
  scheduleId: Scalars['ID'];
}>;


export type DeleteStripeSubscriptionScheduleMutation = { __typename?: 'Mutation', deleteStripeSubscriptionSchedule?: boolean | null };
